# -*- coding: utf-8 -*-
"""
Ceci peut être considéré comme une boite noire. A lire à vos risques et périls.
"""


import tkinter as tk
import tkinter.messagebox as tbox
import sudoku_resolution as sr
import copy

import sys

if sys.platform == 'linux':
    ENCODAGE = 'utf8'
else:
    ENCODAGE = 'latin-1'


class Sudoku(tk.Frame):

    GRIS = "#999999"
    BLANC = 'white'

    STEST = [[0, 0, 0, 7, 1, 0, 0, 0, 0],
             [0, 0, 8, 0, 6, 0, 3, 0, 0],
             [0, 0, 0, 0, 5, 0, 0, 4, 0],
             [6, 8, 0, 0, 0, 0, 0, 0, 2],
             [3, 0, 9, 0, 2, 6, 0, 0, 0],
             [0, 0, 0, 0, 0, 9, 0, 5, 7],
             [8, 1, 0, 4, 0, 3, 0, 0, 0],
             [4, 9, 0, 0, 0, 0, 2, 0, 8],
             [7, 5, 0, 0, 8, 0, 4, 0, 3]]

    def __init__(self, sbase=STEST, master=None):
        tk.Frame.__init__(self, master=master)
        self.pack()
        self.createWidgets()
        self.set_coeffs(sbase, c=True)

    def createWidgets(self):
        # variables liées au données
        self.vars = [[tk.StringVar() for j in range(9)] for i in range(9)]

        for i in range(9):
            ligne = tk.Frame(self)  # un cadre par ligne
            ligne.pack(expand=1)  # chaque ligne rempli tout l'espace
            for j in range(9):
                # couleur qui alterne entre "carré"
                color = self.BLANC
                I = i // 3
                J = j // 3
                if (I + J) % 2 == 0:
                    color = self.GRIS
                # création d'une case par element dans la ligne i de la matrice
                tk.Entry(
                    ligne,
                    textvariable=self.vars[i][j],
                    width=5,
                    justify='center',
                    bg=color
                ).pack(side='left')  # et non pas right
                # ajout d'un callback pour toute écriture des variables
                # 'w' est le mode d'écoute (write)
                # on passe une fonction anonyme comme callback
                self.vars[i][j].trace(
                    'w',
                    lambda name, index, mode, var=self.vars[i][j], i=i, j=j:
                        self.change_coeff(var, i, j))
        ligne_btns = tk.Frame(self)
        ligne_btns.pack(side='bottom')

        resoudre = tk.Button(ligne_btns, text="Résoudre".encode(ENCODAGE),
                             command=self.resoudre)
        resoudre.pack(anchor='center', side="left", padx=5, pady=5)

        fiable = tk.Button(ligne_btns, text="Fiabilité".encode(ENCODAGE),
                           command=self.fiable)
        fiable.pack(anchor='center', side="left", padx=5, pady=5)

        res = tk.Button(ligne_btns, text='Reset', command=self.reset)
        res.pack(anchor='center', side="left", padx=5, pady=5)

    def set_coeffs(self, s, c=False):
        if c:
            self.s_base = copy.deepcopy(s)
            self.s_copy = copy.deepcopy(s)
        for i in range(9):
            for j in range(9):
                if self.s_copy[i][j] != 0:
                    self.vars[i][j].set(str(s[i][j]))
                else:
                    self.vars[i][j].set('')

    def resoudre(self):
        try:
            if not sr.fiable(self.s_copy):
                tbox.showwarning('Echec',
                                 'Sudoku invalide, chiffre répété !'
                                 .encode(ENCODAGE))
                res = -2  # on ne fait rien...
            else:
                res = sr.resolution(self.s_copy)  # on appelle resolution de
            #                                   sudoku_resolution
        except:
            tbox.showerror('Erreur',
                           'Voir la console pour plus de détails'
                           .encode(ENCODAGE))
            raise
        if res == 1:
            self.set_coeffs(self.s_copy)
        elif res == 0:
            tbox.showwarning('Echec', 'Aucune solution !')
        elif res == -1:
            tbox.showwarning('Attention', 'Sudoku déjà rempli !')
        else:
            pass

    def fiable(self):
        if sr.fiable(self.s_copy):
            tbox.showwarning('Vérification', 'Pas de problème détecté')
        else:
            tbox.showwarning('Echec', 'Sudoku invalide, chiffre répété !')

    def reset(self):
        self.set_coeffs(self.s_base, c=True)
        return self

    def change_coeff(self, var, i, j):
        """
        Change la grille de données pour refléter un changement à l'écran
        """
        val = var.get()
        if val == '':
            self.s_copy[i][j] = 0
        else:
            self.s_copy[i][j] = int(val)

if __name__ == '__main__':
    root = tk.Tk()
    s = Sudoku(master=root)
    # s.set_coeffs(stest, c=True)
    s.mainloop()
